﻿using System;
using System.Collections.Generic;
using System.Linq;
using System.Text;
using System.Windows;
using System.Windows.Controls;
using System.Windows.Data;
using System.Windows.Documents;
using System.Windows.Input;
using System.Windows.Media;
using System.Windows.Media.Imaging;
using System.Windows.Navigation;
using System.Windows.Shapes;
using System.ComponentModel;
using System.Collections.ObjectModel;
using Infragistics.Windows.DataPresenter;
using System.Globalization;

namespace HeatMapProblem
{

    public partial class MainWindow : Window, INotifyPropertyChanged
    {
        private ObservableCollection<Account> _Accounts;
        public ObservableCollection<Account> Accounts
        {
            get { return _Accounts; }
            set
            {
                if (_Accounts != value)
                {
                    _Accounts = value;
                    OnPropertyChanged("Accounts");
                }
            }
        }

        private decimal _MinAccountBalance;
        public decimal MinAccountBalance
        {
            get { return _MinAccountBalance; }
            set
            { 
                if(_MinAccountBalance != value)
                {
                    _MinAccountBalance = value;
                    OnPropertyChanged("MinAccountBalance");
                }
            }
        }

        private decimal _MaxAccountBalance;
        public decimal MaxAccountBalance
        {
            get { return _MaxAccountBalance; }
            set
            {
                if (_MaxAccountBalance != value)
                {
                    _MaxAccountBalance = value;
                    OnPropertyChanged("MaxAccountBalance");
                }
            }
        }

        private Random random;

        public MainWindow()
        {
            InitializeComponent();
            this.DataContext = this;
            random = new Random();
            CreateFakeData();
        }


        private void RefreshGrids_Click(object sender, RoutedEventArgs e)
        {
            // We need to prevent the grid from re-binding until the Accounts list is built AND Min/Max Balances are computed.
            Grid1.ClearValue(XamDataGrid.DataSourceProperty); // this statement does nothing.  Grid still points to Accounts list and attempts to bind.
            Grid2.ClearValue(XamDataGrid.DataSourceProperty); // this statement does nothing.  Grid still points to Accounts list and attempts to bind.
            CreateFakeData();
            Grid1.DataSource = Accounts;
            Grid2.DataSource = Accounts;
        }


        public void CreateFakeData()
        {
            Accounts = new ObservableCollection<Account>();

            for (int i = 0; i < 20; i++)
            {
                Account account = new Account { AccountName = "Account " + i.ToString(), AccountBalance = random.Next(1000000, 100000000) };
 
                for (int j = 0; j < 10; j++)
                    account.Positions.Add(new Position {SecurityName = "Security " + j.ToString(), Coupon= random.Next(2,15), Quantity = random.Next(1000,100000) });

                Accounts.Add(account);
            }

            MinAccountBalance = Accounts.Min(x => x.AccountBalance); // Grid attempts to rebind here, causing out-of-range erros in the converter
            MaxAccountBalance = Accounts.Max(x => x.AccountBalance);
            List<decimal> zz = Accounts.Select(x => x.AccountBalance).OrderBy(x => x).ToList();
        }


        private void Grid2_FieldLayoutInitialized(object sender, Infragistics.Windows.DataPresenter.Events.FieldLayoutInitializedEventArgs e)
        {
            // Generate a distinct list of coupons.  A column will be created for each coupon.
            List<decimal> coupons = Accounts.SelectMany<Account, Position>(x => x.Positions).Select(x => x.Coupon).Distinct().OrderBy(x => x).ToList();

            // Find the min/max value of all positions
            decimal MinValue = Accounts.SelectMany<Account, Position>(x => x.Positions).Min(x => x.Quantity);
            decimal MaxValue = Accounts.SelectMany<Account, Position>(x => x.Positions).Max(x => x.Quantity);


            foreach (decimal coupon in coupons)
            {
                string columnName = Guid.NewGuid().ToString();      // Generate a bogus field name 

                UnboundField unboundField = new UnboundField
                {
                    Name = columnName,
                    Label = coupon == -1 ? "Total" : coupon.ToString("#.###") + "%",
                    DataType = typeof(decimal)
                };

                MultiBinding mb = new MultiBinding { Converter = new CouponColumnConverter() };
                mb.Bindings.Add(new Binding());                             // The row being bound
                mb.Bindings.Add(new Binding { Source = coupon });           // The coupon that identifies the column. 
                mb.Bindings.Add(new Binding { Source = MinValue });         // Min/Max for heatmap
                mb.Bindings.Add(new Binding { Source = MaxValue });
                unboundField.Binding = mb;
                e.FieldLayout.Fields.Add(unboundField);
            }
        }

        #region INotifyPropertyChanged Members
        public event PropertyChangedEventHandler PropertyChanged;

        public void OnPropertyChanged(string name)
        {
            if (PropertyChanged != null)
            {
                PropertyChanged(this, new PropertyChangedEventArgs(name));
            }
        }
        #endregion

    }

    public class CouponColumnConverter : IMultiValueConverter
    {
        public object Convert(object[] value, Type targetType, object parameter, System.Globalization.CultureInfo culture)
        {
            // Get parameters passed in the binding
            Account boundItem = (Account)value[0];                  // The account row being bound
            decimal coupon = System.Convert.ToDecimal(value[1]);    // Coupon of the column being bound
            decimal minValue = (decimal)value[2];                   // Min/max for heatmap
            decimal maxValue = (decimal)value[3];
            decimal result = 0;


            // Here we get the value which is to be displayed in the cell.  We look at the passed coupon
            // and find all positions where the coupon property matches the passed coupon.  If no 
            // positions are found we just leave the cell blank (Binding.DoNothing).
            // At this point we can compute a value for the heat map 


            if (boundItem.Positions != null && boundItem.Positions.Any())
            {
                result = boundItem.Positions.Where(x => x.Coupon == coupon).Sum(x => x.Quantity);
            }

            if (result == 0)
                return Binding.DoNothing;
            else
                return result;
        }

        public object[] ConvertBack(object value, Type[] targetType, object parameter, System.Globalization.CultureInfo culture)
        {
            throw new NotImplementedException();
        }
    }

    public class HeatMapConverter : IMultiValueConverter
    {
        public object Convert(object[] values, Type targetType, object parameter, CultureInfo culture)
        {
            if (values[0] == null || values[0] == DependencyProperty.UnsetValue )
                return Binding.DoNothing;

            decimal value = (decimal)values[0];
            decimal min = (decimal)values[1];
            decimal max = (decimal)values[2];
            decimal val = (max - min) == 0 ? 1 : (value - min) / (max - min);

            if (val > 1) val = 1;
            if (val < 0) val = 0;

            return new System.Windows.Media.SolidColorBrush(
                new System.Windows.Media.Color
                {
                    A = 255,
                    R = System.Convert.ToByte(255 * val),
                    G = System.Convert.ToByte(255 * (1 - val)),
                    B = 0
                });
        }

        public object[] ConvertBack(object value, Type[] targetTypes, object parameter, CultureInfo culture)
        {
            throw new NotImplementedException();
        }
    }

    public class Position
    {
        public string SecurityName { get; set; }
        public decimal Coupon { get; set; }
        public decimal Quantity { get; set; }
    }

    public class Account
    {
        public string AccountName { get; set; }
        public decimal AccountBalance { get; set; }
        public List<Position> Positions { get; set; }

        public Account()
        {
            Positions = new List<Position>();
        }
    }
}
